from requests import Session
from dorothy_sdk.utils import url_join


class QualityAnnotation:
    resource = "annotation"
    project_id: str = None
    under_penetrated: bool = None
    over_penetrated: bool = None
    costophrenic_cropped: bool = None
    apices_cropped: bool = None
    reliable_radiography: bool = None
    minimum_interpretation_quality: bool = None

    def __init__(self, session: Session, host: str, *args, **kwargs):
        self._session: Session = session
        self._service_host = host
        if kwargs.get("project_id"):
            self.project_id = kwargs.get("project_id")
        if kwargs.get("under_penetrated"):
            self.under_penetrated = kwargs.get("under_penetrated")
        if kwargs.get("over_penetrated"):
            self.over_penetrated = kwargs.get("over_penetrated")
        if kwargs.get("costophrenic_cropped"):
            self.costophrenic_cropped = kwargs.get("costophrenic_cropped")
        if kwargs.get("apices_cropped"):
            self.apices_cropped = kwargs.get("apices_cropped")
        if kwargs.get("reliable_radiography"):
            self.reliable_radiography = kwargs.get("reliable_radiography")
        if kwargs.get("minimum_interpretation_quality"):
            self.minimum_interpretation_quality = kwargs.get("minimum_interpretation_quality")

    def get(self, project_id: str = None):
        search_parameter = {}
        if project_id or self.project_id:
            search_parameter.update({"project_id": project_id or self.project_id})
        response = self._session.get(url_join(self._service_host, self.resource), params=search_parameter)
        if response.status_code == 200:
            return response.json()
        elif response.status_code == 404:
            raise ValueError("There are no annotations for this project_id")
        else:
            response.raise_for_status()

    def save(self, under_penetrated: bool = None,
             over_penetrated: bool = None,
             costophrenic_cropped: bool = None,
             apices_cropped: bool = None,
             reliable_radiography: bool = None,
             minimum_interpretation_quality: bool = None):
        if not self.project_id:
            raise ValueError("'project_id' is a mandatory key")
        data = {
            'project_id': self.project_id,
            'under_penetrated': self.under_penetrated or under_penetrated,
            'over_penetrated': self.over_penetrated or over_penetrated,
            'costophrenic_cropped': self.costophrenic_cropped or costophrenic_cropped,
            'apices_cropped': self.apices_cropped or apices_cropped,
            'reliable_radiography': self.reliable_radiography or reliable_radiography,
            'minimum_interpretation_quality': self.minimum_interpretation_quality or minimum_interpretation_quality
        }
        data = {key: value for key, value in data.items() if value is not None}
        response = self._session.post(
            url=url_join(self._service_host, self.resource) + '/',
            json=data
        )
        response.raise_for_status()



from dorothy_sdk.resources.dataset import Dataset
from dorothy_sdk.resources.image import Image
from dorothy_sdk.resources.quality_annotation import QualityAnnotation
from dorothy_sdk.resources.folders import CrossValidationFolders



import os
import tempfile
from requests import Session
from PIL import Image as PILImage
from io import BytesIO

CACHE_LOCATION = os.path.join(tempfile.gettempdir(), "image_service", "images")
CACHE_ENABLED = False


class Image:
    resource = "images"

    def __init__(self, dataset_name: str, image_url: str, project_id: str,
                 insertion_date: str, metadata: dict, date_acquisition: str, number_reports: int,
                 session: Session, host: str, dataset_id: str = None, *args, **kwargs):
        self.dataset_name = dataset_name
        self.image_url = image_url
        self.project_id = project_id
        self.insertion_date = insertion_date
        self.metadata = metadata
        self.date_acquisition = date_acquisition
        self.number_reports = number_reports
        self.dataset_id = dataset_id
        self._session: Session = session
        self._service_host = host
        self.image_object: PILImage = None
        self.image_format = None
        self.download_parameters = None
        self._cache_location = CACHE_LOCATION
        self._cached_image_name = f"{self.dataset_name}_{self.project_id}.{self.image_format}"

    def _check_cache(self) -> [PILImage, None]:
        if CACHE_ENABLED:
            file_name = f"{self.dataset_name}_{self.project_id}.{self.image_format}"
            image_cached_path = os.path.join(self._cache_location, file_name)
            if not os.path.exists(image_cached_path):
                return None
            return PILImage.open(image_cached_path, formats=self.image_format)
        return None

    def _update_cache(self, image_content: bytes):
        if CACHE_ENABLED:
            if not os.path.exists(self._cache_location):
                os.makedirs(self._cache_location, exist_ok=True)
            with open(os.path.join(self._cache_location, self._cached_image_name), mode="w") as file:
                file.write(image_content)
        return None

    def set_cache_location(self, path: str) -> None:
        if CACHE_ENABLED:
            if not os.path.exists(path):
                os.makedirs(path, exist_ok=True)
            self._cache_location = path

    def download_image(self, width: int = None, height: int = None, gray_scale: bool = False) -> PILImage:
        cached_image = self._check_cache()
        if not cached_image:
            parameters = {}
            if width:
                parameters["width"] = width
            if height:
                parameters["height"] = height
            parameters["grayscale"] = gray_scale
            self.download_parameters = parameters
            request = self._session.get(self.image_url, params=parameters)
            request.raise_for_status()
            self.image_format = request.headers.get("Content-Type", "image/png").split("/")[-1]
            self._update_cache(request.content)
            if request.status_code == 200:
                buffer = BytesIO()
                buffer.write(request.content)
                buffer.seek(0)
                image_object = PILImage.open(buffer)
                self.image_object = image_object
                return image_object
            else:
                raise RuntimeError("Could not download image")
        return cached_image

    def save(self, image_path: str = None, quality: int = 100) -> None:
        if not self.image_object:
            self.download_image()
        if image_path:
            self.image_object.save(image_path, quality=quality)
            return
        else:
            self.image_object.save(f"{self.dataset_name}_{self.project_id}.{self.image_format}", quality=quality)
            return

    def crop(self, initial_position: tuple, incremental_size: int) -> PILImage:
        image_width, image_height = self.image_object.size
        if incremental_size > image_width or incremental_size > image_height:
            raise ValueError("The crop range exceeds the maximum image size. Reduce 'incremental_size' to solve it")
        final_position = tuple(map(lambda x: x + incremental_size, initial_position))
        box = initial_position + final_position
        return self.image_object.crop(box)

    def flip(self):
        return self.image_object.transpose(PILImage.FLIP_LEFT_RIGHT)

    def scale_width(self, target_size: int, crop_size: int, method=PILImage.BICUBIC) -> PILImage:
        image_width, image_height = self.image_object.size
        if image_width == target_size and image_height >= crop_size:
            return self.image_object
        new_width = target_size
        new_height = int(max(target_size * image_height / image_width, crop_size))
        return self.image_object.resize((new_width, new_height), method)

    def make_power_2(self, base, method=PILImage.BICUBIC) -> PILImage:
        image_width, image_height = self.image_object.size
        new_height = int(round(image_height / base) * base)
        new_width = int(round(image_width / base) * base)

        if new_height == image_height and new_width == image_width:
            return self.image_object

        return self.image_object.resize((new_width, new_height), method)



import os
from requests_cache import CachedSession
from datetime import timedelta
from requests.adapters import HTTPAdapter, Retry

DEFAULT_TIMEOUT = 60  # seconds


class TimeoutHTTPAdapter(HTTPAdapter):
    def __init__(self, *args, **kwargs):
        self.timeout = DEFAULT_TIMEOUT
        if "timeout" in kwargs:
            self.timeout = kwargs["timeout"]
            del kwargs["timeout"]
        super().__init__(*args, **kwargs)
        self.max_retries = Retry(
            total=3,
            backoff_factor=0.1,
            status_forcelist=[500, 502, 503, 504],
        )

    def send(self, request, **kwargs):
        timeout = kwargs.get("timeout")
        if timeout is None:
            kwargs["timeout"] = self.timeout
        return super().send(request, **kwargs)


class SessionManager:
    _token = None
    _token_file = os.path.join(os.getcwd(), "dorotysdk", "credentials.txt")
    _environment_variable_name = "DOROTYSDK_ACCESS_TOKEN"

    def __init__(self, token: str = None, **options):
        super().__init__()
        if not token:
            self._fetch_token(**options)
        else:
            self._token = token

    def build_session(self):
        session = CachedSession(
            'dorothy_sdk',
            use_cache_dir=True,  # Save files in the default user cache dir
            cache_control=True,  # Use Cache-Control headers for expiration, if available
            expire_after=timedelta(hours=6),  # Otherwise expire responses after one day
            allowable_methods=['GET', 'POST'],  # Cache POST requests to avoid sending the same data twice
            allowable_codes=[200, 400],  # Cache 400 responses as a solemn reminder of your failures
            match_headers=True,  # Match all request headers
            stale_if_error=True,  # In case of request errors, use stale cache data if possible
        )
        session.headers.update({"Authorization": f"Token {self._token}"})
        adapter = TimeoutHTTPAdapter()
        session.mount("https://", adapter)
        session.mount("http://", adapter)
        return session

    def _fetch_token(self, **options):
        if options.get('path', ''):
            if os.path.exists(options.get('path')):
                self._token_file = options.get('path')
            else:
                FileNotFoundError("Credentials file passed as parameter not found.")

        token = os.environ.get(self._environment_variable_name, '')
        if not token:
            if os.path.exists(self._token_file):
                with open(self._token_file, mode='r', encoding='utf-8') as credentials_file:
                    self._token = credentials_file.read().replace('\n', '').replace('\t', '').replace('\r', '').strip()
            else:
                raise RuntimeError("Access credential not found. The credential can be entered either as a parameter "
                                   "or via a credentials file or environment variable")
        else:
            self._token = token



import warnings
from typing import List

from dorothy_sdk.resources import Dataset, Image, QualityAnnotation, CrossValidationFolders
from dorothy_sdk.session import SessionManager
from dorothy_sdk.utils import url_join


class Client:
    _service_host: str = "https://dorothy-image.lps.ufrj.br"

    def __init__(self, token=None, **kwargs):
        self.session = SessionManager(token, **kwargs).build_session()
        if kwargs.get("host"):
            warnings.warn(f"Modifying host from {self._service_host} to {kwargs.get('host')}")
            self._service_host = kwargs.get("host")

    def get_datasets(self) -> List[Dataset]:
        request = self.session.get(url_join(self._service_host, Dataset.resource))
        request.raise_for_status()
        datasets = []
        for element in request.json():
            datasets.append(Dataset(session=self.session, host=self._service_host, **element))
        return datasets

    def dataset(self, dataset_id: str) -> Dataset:
        request = self.session.get(url_join(self._service_host, Dataset.resource), params={"search": dataset_id})
        request.raise_for_status()
        if request.status_code == 200:
            return Dataset(session=self.session, host=self._service_host,
                           **[dataset for dataset in request.json() if dataset['name'] == dataset_id][0])
        elif request.status_code == 404:
            return None
        else:
            raise RuntimeError("Unable to fetch dataset")

    def image(self, image_id: str) -> Image:
        request = self.session.get(url_join(self._service_host, Image.resource), params={"search": image_id})
        request.raise_for_status()
        if request.status_code == 200:
            return Image(session=self.session, host=self._service_host, **request.json()[0])
        elif request.status_code == 404:
            return None
        else:
            raise RuntimeError("Unable to fetch image")

    def quality_annotations(self, project_id: str = None) -> QualityAnnotation:
        return QualityAnnotation(session=self.session, host=self._service_host, project_id=project_id)

    def create_quality_annotation(self, project_id: str, under_penetrated: bool = None,
                                  over_penetrated: bool = None,
                                  costophrenic_cropped: bool = None,
                                  apices_cropped: bool = None,
                                  reliable_radiography: bool = None,
                                  minimum_interpretation_quality: bool = None):
        QualityAnnotation(session=self.session, host=self._service_host, project_id=project_id).save(
            under_penetrated, over_penetrated, costophrenic_cropped, apices_cropped,
            reliable_radiography, minimum_interpretation_quality
        )

    def list_quality_annotations(self):
        return QualityAnnotation(session=self.session, host=self._service_host, project_id=None).get()

    def list_clusters(self):
        return CrossValidationFolders(session=self.session, host=self._service_host).get()

    def cluster(self, cluster_id: int):
        return CrossValidationFolders(session=self.session, host=self._service_host, cluster_id=cluster_id)



from requests import Session
from dorothy_sdk.utils import url_join


class CrossValidationFolders:
    resource = "cross_validation/cluster/"
    dataset: str = None
    cluster_id: str = None
    file_url: str = None

    def __init__(self, session: Session, host: str, **kwargs):
        self._session: Session = session
        self._service_host = host
        if kwargs.get("dataset"):
            self.dataset = kwargs.get("dataset")
        if kwargs.get("cluster_id"):
            self.cluster_id = kwargs.get("cluster_id")

    def get(self, cluster_id: str = None, dataset: str = None):
        search_parameter = {}
        if cluster_id or self.cluster_id:
            search_parameter.update({"cluster_id": cluster_id or self.cluster_id})
        if dataset or self.dataset:
            search_parameter.update({"dataset": dataset or self.dataset})
        response = self._session.get(url_join(self._service_host, self.resource), params=search_parameter)
        if response.status_code == 200:
            body = response.json()
            if isinstance(body, dict):
                self.file_url = body.get("file_url")
            if cluster_id and not self.cluster_id:
                self.cluster_id = cluster_id
            if dataset and not self.dataset:
                self.dataset = dataset
            return body
        elif response.status_code == 404:
            raise ValueError("There are no folders for this dataset or cluster id not found.")
        else:
            response.raise_for_status()

    def download_file(self, path: str = None):
        if not self.file_url and not self.cluster_id:
            raise ValueError("Cluster id is mandatory to download file")
        if not self.file_url and self.cluster_id:
            self.get()
        if self.file_url:
            request = self._session.get(self.file_url)
            request.raise_for_status()
            if request.status_code == 200:
                if path:
                    with open(path, mode="r+") as file:
                        file.write(request.content)
                    return
                return request.content
            else:
                raise RuntimeError("Could not download image")



def url_join(*args):
    url_args = []
    for element in args:
        if not isinstance(element, str):
            raise RuntimeError("Only elements of type 'str' are accepted.")
        else:
            if element.endswith("/"):
                element = element[:-1]
            if element.startswith("/"):
                element = element[1:]
            url_args.append(element)
    return "/".join(url_args)




import os
import platform
from os.path import join, abspath, dirname
from setuptools import find_packages, setup


def get_version() -> str:
    """Get the package version
    Returns:
        [str]: The package version
    """
    global_names = {}
    exec(  # pylint: disable=exec-used
        open(
            join(
                dirname(abspath(__file__)),
                "dorothy_sdk",
                "version.py"
            )
        ).read(),
        global_names
    )
    return global_names["__version__"]


with open("README.rst", "r") as fh:
    long_description = fh.read()

PACKAGE_NAME = "dorothy-sdk"
PACKAGE_DESCRIPTION = ""
PACKAGE_DOWNLOAD_URL = None
PACKAGE_AUTHOR = "Patrick Braz"
PACKAGE_AUTHOR_EMAIL = "patrickfbraz@poli.ufrj.br"
PACKAGE_MAINTAINER = "Patrick Braz"
PACKAGE_EMAIL = "patrickfbraz@poli.ufrj.br"

setup(
    name=PACKAGE_NAME,
    url="https://github.com/tb-brics/dorothy-sdk",
    description=PACKAGE_DESCRIPTION,
    long_description=long_description,
    version=get_version(),
    download_url=PACKAGE_DOWNLOAD_URL,
    author=PACKAGE_AUTHOR,
    author_email=PACKAGE_AUTHOR_EMAIL,
    maintainer=PACKAGE_MAINTAINER,
    maintainer_email=PACKAGE_EMAIL,
    install_requires=["requests-cache==0.9.5"],
    packages=find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.6",
    package_dir={
        "": ".\\" if platform.platform() == "Windows" else "./"
    },
    include_package_data=True
)



from typing import List
from requests import Session
from dorothy_sdk.utils import url_join
from dorothy_sdk.resources.image import Image


class Dataset:
    resource = "datasets"

    def __init__(self, name: str, image_formats: str, number_images: int, session: Session, host: str, *args, **kwargs):
        self.number_images = number_images
        self.name = name
        self.image_formats = image_formats
        self._session = session
        self._service_host = host

    def list_images(self) -> List[Image]:
        request = self._session.get(url_join(self._service_host, Image.resource), params={"search": self.name})
        request.raise_for_status()
        if request.status_code == 200:
            return [Image(session=self._session, host=self._service_host, **element) for element in request.json()]
        else:
            raise RuntimeError("Unable to list available images")

    def get_image(self, image_id: str) -> Image:
        request = self._session.get(url_join(self._service_host, Image.resource), params={"search": f"{self.name},{image_id}"})
        if request.status_code == 200:
            return Image(session=self._session, host=self._service_host, **request.json())
        elif request.status_code == 404:
            return None
        else:
            raise RuntimeError("Unable to fetch image")
